#!/usr/bin/perl -w

#Copyright (c) Nmap project and its contributor
#All rights reserved.

#Redistribution and use in source and binary forms, with or without 
#modification, are permitted provided that the following conditions 
#are met:

# 1. Redistributions of source code must retain the above copyright 
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in 
#    the documentation and/or other materials provided with the 
#    distribution.
# 3. Neither the name of Nmap nor the names of its contributors 
#    may be used to endorse or promote products derived from this 
#    software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.


package Nmap::CGI::Auth;

require Exporter;
@ISA 	= qw( Exporter );
@EXPORT = qw( regauth authuser delete_auth);

use strict;
use CGI::Session;
use CGI::Cookie;
use Nmap::Main;


sub regauth
{
	my ( $user , $session );

	$user = shift;

	$session = new CGI::Session( "driver:File" , undef , {Directory=>'/tmp'} );
	$session->expires("+2d");
	$session->param( 'username' , $user->get( 'username' ) );
	return( CGI->cookie(session => $session->id ) );
}

sub isauth
{
	my ( $sid );
	$sid = CGI->cookie( 'session' );

	return( 1 ) if( $sid );
	return( 0 );
}

sub authuser
{
	my ( $main , $sid , $user , $uname , $session );

	$main = shift;

	$sid = CGI->cookie( 'session' );
	$session = new CGI::Session(undef, $sid, {Directory=>'/tmp'});
	$uname = $session->param( 'username' );

	return undef if( ! defined $sid );
	return undef if( ! defined $uname );
	debug( "sid: " . $sid  . "\n");
	debug( "username " . $uname );

	$user = new Nmap::User( $main , $uname );

	return undef if( ! defined( $user ) );
	return undef if( $user->getdb != 1 );

	return( $user );
}

sub delete_auth
{
	my( $sid , $session );
	$sid = CGI->cookie( 'session' );
	$session = new CGI::Session(undef, $sid, {Directory=>'/tmp'});
	$session->delete() if( defined $session );
}

1;
__END__

=head1 SYNOPSIS

Nmap::CGI::Auth handle authentification for the web-app in nmap-cgi project

=head1 METHODS

=head2 delete_auth( )

Delete all authentification informations from the current session.

=head2 authuser( Nmap::Main )

Returns the user currently logged on the web-app. 
If a user is logged and exists, returns the Nmap::User
instance. If no user is logged or if it does not exists,
returns undef.

=head2 regauth( Nmap::User )

Register a user in the current session. The registered user
is the parameter. It _must_ be a Nmap::User reference.

=head2 isauth()

Returns 1 if a user is logged, 0 otherwise. It does not
check if the user is valid or not.

=head1 SEE ALSO

Nmap::User

=head1 AUTHOR

Julien Delange <julien at gunnm dot org>

=head1 COPYRIGHT

Julien Delange <julien at gunnm dot org>

