#!/usr/bin/perl -w 

package Nmap::Conf;

use strict;
use Config::General;

#Copyright (c) Nmap project and its contributor
#All rights reserved.

#Redistribution and use in source and binary forms, with or without 
#modification, are permitted provided that the following conditions 
#are met:

# 1. Redistributions of source code must retain the above copyright 
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in 
#    the documentation and/or other materials provided with the 
#    distribution.
# 3. Neither the name of Nmap nor the names of its contributors 
#    may be used to endorse or promote products derived from this 
#    software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.


my $filename;

sub new
{
    my $class;
    my $self;
	my $this;
	my $param;
	my $conf;
my %config;

	$this = shift;
	$param = shift;
    
	$class = ref($this) || $this;

	$self = {};
    $conf = new Config::General( $param );
	%config = $conf->getall;
	$self->{'data'} = \%config;
    bless($self , $class );
    return $self;
}


sub get_themes
{
	my $self = shift;
	my @ret;
	@ret = split( /[\s+|\t+]+/ , $self->{'data'}->{'config'}{'themes'} );
	return( \@ret );
}

sub get_default_theme
{
	my $self = shift;
	return( $self->{'data'}->{'config'}{'default-theme'} );
}

sub get_dbuser
{
	my $self = shift;
	return( $self->{'data'}->{'config'}{'dbuser'} );
}

sub get_datadir
{
	my $self = shift;
	return( $self->{'data'}->{'config'}{'datadir'} );
}

sub get_dbpass
{
	my $self = shift;
	return( $self->{'data'}->{'config'}{'dbpass'} );
}

sub get_dbname
{
	my $self = shift;
	return( $self->{'data'}->{'config'}{'dbname'} );
}

sub get_dbport
{
	my $self = shift;
	return( $self->{'data'}->{'config'}{'dbport'} );
}

sub get_dbhost
{
	my $self = shift;
	return( $self->{'data'}->{'config'}{'dbhost'} );
}

sub get_open
{
	my $self = shift;
	if( ( defined( $self->{'data'}->{'config'}{'open'} ) ) &&
		( $self->{'data'}->{'config'}{'open'} eq 'true' ) )
		{
			return( 1 );
		}
		return( 0 );
}

sub get_subopen
{
	my $self = shift;
	if( ( defined( $self->{'data'}->{'config'}{'subopen'} ) ) &&
		( $self->{'data'}->{'config'}{'subopen'} eq 'true' ) )
		{
			return( 1 );
		}
		return( 0 );
}

sub get_mailfrom
{
	my $self = shift;
	if( defined( $self->{'data'}->{'config'}{'mailfom'} ) ) 
	{
		return( $self->{'data'}->{'config'}{'mailfom'} );
	}
	return( "nmap\@localhost" );
}


1;

__END__

=head1 SYNOPSIS

Nmap::Conf handle config file for nmap-cgi project

=head1 DESCRIPTION

This module is an object which handle the configuration of nmap-cgi.

=head1 METHODS

Except new() and get_themes() method, all other method returns a scalar which contains the value of a configuration directive. For exemple, get_dbport returns the value of the dbport directive.
If the value is not defined, it returns undef.

=head2 new

Create a new instance of this object. Only one argument is required : the file which contains the config. Use as it :

my $conf = new Nmap::Conf( "/etc/nmap-cgi/nmap-cgi.conf" );

=head2 get_mailfrom()

Returns the mailfrom configuration directive. Undef if the variable is not defined.

=head2 get_subopen()

Returns the subopen configuration directive. Returns undef if not defined.

=head2 get_dbhost()

=head2 get_dbport()

=head2 get_dbname()

=head2 get_dbuser()

=head2 get_dbpass()

=head2 get_datadir()

=head2 get_default_theme()

=head2 get_themes()

Returns an arrayref where the array contains the name of the available themes declared in the config file.


=head1 EXAMPLE
	
$conf = Nmap::Conf( "/etc/nmap-cgi/nmap-cgi.conf" )

print "dbport directive is set to: " . $conf->get_dbport;

print "Themes : " ;

foreach ( @{$conf->get_themes} )

{

	print $_;

}

=head1 SEE ALSO

Nmap::Main

=head1 AUTHOR

Julien Delange <julien at gunnm dot org>

=head1 COPYRIGHT

Julien Delange <julien at gunnm dot org>

