#!/usr/bin/perl

#Copyright (c) Nmap project and its contributor
#All rights reserved.

#Redistribution and use in source and binary forms, with or without 
#modification, are permitted provided that the following conditions 
#are met:

# 1. Redistributions of source code must retain the above copyright 
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in 
#    the documentation and/or other materials provided with the 
#    distribution.
# 3. Neither the name of Nmap nor the names of its contributors 
#    may be used to endorse or promote products derived from this 
#    software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.

package Nmap::Functions;

require Exporter;
@ISA 	= qw(Exporter);
@EXPORT = qw(gen_pwd gen_confirm addslashes stripslashes);

use strict;


# When you fetch data from the database, some caracters
# have been added to prevent SQL injection attacks
# This function removes all added caracters
sub stripslashes
{
    my $string = shift;
	return( undef ) if( ! defined( $string ) );

    $string =~ s/\\//g;
    $string =~ s/\Q//g;

    return ( $string  );
}

# This function prevent SQL injection
# You give as the first argument a data
# and escape all malicious caracter
sub addslashes
{
        my $string = shift;
        return( quotemeta( $string ) );
}

#Generate a random password
sub gen_pwd
{
	my $password;
	for (0 .. 7) { $password .= ('a'..'z', 'A'..'Z', '0'..'9')[int rand 62] ; }
	return $password;
}

sub gen_confirm
{
	my $code;
	for (0 .. 16) { $code .= ('a'..'z', 'A'..'Z', '0'..'9')[int rand 62] ; }
	return $code;
}


1;


__END__

=head1 SYNOPSIS

Nmap::Functions some functions useful to nmap-cgi program.

=head1 DESCRIPTION

This module contains a lot of functions used in the nmap-cgi program and that cannot be in other modules.

=head1 METHODS

=head2 addslashes( SCALAR )

Add some escaped caracters to prevent SQL injection attacks. Take as argument a scalar which contains a string and returns the same string with special caracters espaced.

=head2 stripslashes( SCALAR )

Contrary of addslashes. Take as argument a scalar which contains a string and returns this string without escaped caracters.

=head2 gen_pwd()

Returns a random password. Used to generate new password for new users.

=head2 get_confirm()

Returns a confirmation code when a user subscribe.

=head1 SEE ALSO

Nmap::Main

=head1 AUTHOR

Julien Delange <julien at gunnm dot org>

=head1 COPYRIGHT

Julien Delange <julien at gunnm dot org>

