#!/usr/bin/perl -w
#Copyright (c) Nmap project and its contributor
#All rights reserved.

#Redistribution and use in source and binary forms, with or without 
#modification, are permitted provided that the following conditions 
#are met:

# 1. Redistributions of source code must retain the above copyright 
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in 
#    the documentation and/or other materials provided with the 
#    distribution.
# 3. Neither the name of Nmap nor the names of its contributors 
#    may be used to endorse or promote products derived from this 
#    software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.


package Nmap::Mail;
require Exporter;
@ISA 	= qw( Exporter );
@EXPORT = qw( sendmail );

use strict;
use MIME::Lite;

my $from = "nmap\@localhost";

sub sendmail
{
	my $txt;
	my $line;
	my $key;
	my $tmp;
	my $opts;
	my $msg;
	my $pass;
	
	$opts = shift;

	$opts->{from} = $from if( ! defined( $opts->{from} ) );
	$opts->{subject} = "nmap platform" if( ! defined( $opts->{subject} ) );

	return( -1 ) if( ! defined( $opts->{to} ) );
	return( -2 ) if( ! defined( $opts->{mailname} ) );

	open FIC , "/etc/nmap-cgi/mail-" . $opts->{mailname} or return -3;

	while( $line = <FIC> )
	{
		foreach $key ( keys( %{$opts} ) )
		{
			$pass = 1;
			foreach $tmp ( qw( from subject to mailname ) )
			{
				$pass = 0 if( $tmp eq $key );
			}
			$line =~ s/$key/$opts->{$key}/g if( $pass == 1 );
		}
		$txt .= $line;
	}
	close FIC;

	$msg = new MIME::Lite
		From    => $opts->{from}, 
        To      => $opts->{to},    
        Subject => $opts->{subject},
        Type    => 'text/plain; charset=utf-8',  
        Data    => $txt;

	$msg->send;

	return( 1 );
}


__END__

=head1 SYNOPSIS

Nmap::Mail send a mail for nmap-cgi project.

=head1 DESCRIPTION

This module is used to send mail for the nmap-cgi program. It contains only one method and depends on MIME::Lite module.

=head1 METHODS

=head2 sendmail

Used to send mail. Only one argument : a hash which contains options to send the mail. There is the list of the options, which must be passed as hashkey :

- from	:	from: address set in the mail

- subject :	subject of the mail

- mailname	:	template which contains the mail

- to	:	recipient for the mail

The mailname is the name of the mail template. The mail template are stored in file like : /etc/nmap-cgi/mail-*
For example, if the mailname is test, the mail template will be /etc/nmap-cgi/mail-test.
And the template contains some words, and is it matches with a hashkey, it is replaced with the hashvalue.



=head1 EXAMPLE

Nmap::Mail::sendmail( { to=> "test\@foo.com" , from => "foo\@test.com" , mailname => "register" , LOGIN => "test" , subject => "register on nmap-cgi" } );

=head1 SEE ALSO

Nmap::Main

=head1 AUTHOR

Julien Delange <julien at gunnm dot org>

=head1 COPYRIGHT

Julien Delange <julien at gunnm dot org>

