/*
Copyright (c) Nmap project and its contributor
All rights reserved.

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions 
are met:

1. Redistributions of source code must retain the above copyright 
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in 
   the documentation and/or other materials provided with the 
   distribution.
3. Neither the name of Nmap nor the names of its contributors 
   may be used to endorse or promote products derived from this 
   software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
POSSIBILITY OF SUCH DAMAGE.
*/

DROP TABLE groups;
DROP TABLE users;
DROP TABLE usersprefs;
DROP TABLE scans;
DROP TABLE scansopts;
DROP TABLE comments;
DROP TABLE userips;

CREATE TABLE groups
(
	id			SERIAL,
	def			boolean,
	groupname	VARCHAR,
	CONSTRAINT	grouppkey	PRIMARY KEY( id )
)WITH OIDS;

CREATE TABLE users
(
	id			SERIAL,
	username 	VARCHAR,
	passhash 	VARCHAR,
	firstname	VARCHAR,
	lastname	VARCHAR,
	address		VARCHAR,
	mail		VARCHAR,
	chash		VARCHAR,
	theme		VARCHAR,
	language	VARCHAR,
	state		int4,
	gid			int4 REFERENCES groups( id ),
	CONSTRAINT userpkey PRIMARY KEY( id )
)WITH OIDS;

CREATE TABLE group_rights
(
	gid		int4	REFERENCES groups( id ),
	rname	VARCHAR,
	value	VARCHAR
)WITH OIDS;

CREATE TABLE scans
(
	id			SERIAL,
	target		VARCHAR,
	type		int4,
	state		int4,
	create_date	TIMESTAMP WITHOUT TIME ZONE,
	scan_date	TIMESTAMP WITHOUT TIME ZONE,
	father		int4 REFERENCES scans(id),
	uidowner	int4 REFERENCES users( id ),
	CONSTRAINT scanpkey PRIMARY KEY (id)
)WITH OIDS;


CREATE TABLE scan_opts
(
	sid			int4 REFERENCES scans( id ),
	optname		VARCHAR,
	value		VARCHAR
)WITH OIDS;

CREATE TABLE comments
(
	cid			SERIAL,
	sid			int4 REFERENCES scans( id ),
	uid			int4 REFERENCES users( id ),
	post_date	TIMESTAMP WITHOUT TIME ZONE,
	title		VARCHAR,
	comment		VARCHAR
)WITH OIDS;

CREATE TABLE userips
(
	userid		int4 REFERENCES users( id ),
	adr			VARCHAR
)WITH OIDS;

INSERT INTO users( username , passhash , mail , state ) VALUES ( 'admin' , '$1$D/ez6o0J$1t3OIqL3X1y0VWn/jN8YC0' , 'admin@localhost' , '2' );
INSERT INTO groups( def , groupname ) VALUES ( 'T' , 'default group' );
INSERT INTO group_rights( gid , rname , value ) VALUES ( '1' , 'SCAN-TCP' , '' );
INSERT INTO group_rights( gid , rname , value ) VALUES ( '1' , 'SCAN-UDP' , '' );
INSERT INTO group_rights( gid , rname , value ) VALUES ( '1' , 'ALLOW-SCAN-SINGLE' , '' );
INSERT INTO group_rights( gid , rname , value ) VALUES ( '1' , 'ALLOW-SCAN-SCHEDULED' , '' );
INSERT INTO group_rights( gid , rname , value ) VALUES ( '1' , 'ALLOW-SCAN-PERIODIC' , '' );
INSERT INTO group_rights( gid , rname , value ) VALUES ( '1' , 'ALLOW-SCAN-VERSION' , '' );
INSERT INTO group_rights( gid , rname , value ) VALUES ( '1' , 'MAIL-RESULT' , '' );
