#!/usr/bin/perl -w -I/usr/share/nmap-cgi/api/

#Copyright (c) Nmap project and its contributor
#All rights reserved.

#Redistribution and use in source and binary forms, with or without 
#modification, are permitted provided that the following conditions 
#are met:

# 1. Redistributions of source code must retain the above copyright 
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in 
#    the documentation and/or other materials provided with the 
#    distribution.
# 3. Neither the name of Nmap nor the names of its contributors 
#    may be used to endorse or promote products derived from this 
#    software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.

use Nmap::CGI::Main;
use Nmap::User;
use Nmap::Group;
use Nmap::Functions;
use Nmap::Mail;
use Nmap::CGI::Auth;
use Nmap::Scan;

use Nmap::Logger;
use strict;

my @ACTIONS = qw( edit );

#local vars
my $cgi;
my $nmap;
my $text;
my $action;
my $subaction;
my $user;
my $name;
my $type;
my $tmpl;
my $tmp;
my $opts;
my $uid;
my $sort_type;
my $invert;

( $nmap , $cgi , $user ) = init_web( { 'MUST-AUTH' => 1  , 'MUST-ADMIN' => 1} );

$action			= $cgi->param( 'action' );
$type 			= $cgi->param( 'type' );
$uid 			= $cgi->param( 'uid' );
$sort_type 		= $cgi->param( "sort" );
$invert 		= $cgi->param( "invert" );

# If action is not given in the adress, we can print an error
if( ! ( defined $action ) )
{
	nmaplogevent("admin" , "action not defined" );
	output( undef , { WARN => "Action not defined" } );
}
elsif( $action eq 'list' )
{
	if( $type eq 'user' )
	{
		my $users;
		nmaplogevent( 'admin' , 'list users' );	

		#If the name attr exists, the request come from a search
		$name = $cgi->param( 'name' );

		if( defined $name )
		{
			$users = Nmap::User::getall( $nmap , { USERNAME => $name } );
		}
		else
		{
			$users = Nmap::User::getall( $nmap , { ORDERBY => $sort_type , INVERT => $invert } );
		}
		$tmpl = open_template( 'list' , 'user' );
		$tmpl->param( USERS => $users );
		output( $tmpl->output  );
	}
	if( $type eq 'group' )
	{
		my $groups;
		my $i;

		#If the name attr exists, the request come from a search
		$name = $cgi->param( 'name' );

		nmaplogevent( 'group' , 'list' );

		if( defined $name )
		{
			$groups = Nmap::Group::getall( $nmap , { GROUPNAME => $name} );
		}
		else
		{
			$groups = Nmap::Group::getall( $nmap , { ORDERBY => $sort_type , INVERT => $invert } );
		}
		
		$tmpl = open_template( 'list' , 'group' );
		$tmpl->param( GROUPS => $groups ) if( ( defined $groups ) && ( $#{@$groups} >= 0 ) );
		output( $tmpl->output );
	}
	elsif( $type eq 'scan' )
	{
		my $scans;
		nmaplogevent( 'scan' , 'list' );


		if( defined $name )
		{
			$scans = Nmap::Scan::getall( $nmap , { ADDADMIN => 1  , TARGETLIKE => $name , MAKESTR => 1 , NOTYPE => Nmap::Scan::TYPE_INHERITED , INVERT => $invert , ORDERBY => $sort_type } );
		}
		else
		{
			if( defined( $uid ) )
			{
				$scans = Nmap::Scan::getall( $nmap , { ADDADMIN => 1 , MAKESTR => 1 , NOTYPE => Nmap::Scan::TYPE_INHERITED , OWNER => $uid , INVERT => $invert , ORDERBY => $sort_type } );
			}
			else
			{
				$scans = Nmap::Scan::getall( $nmap , { ADDADMIN => 1 , MAKESTR => 1 , NOTYPE => Nmap::Scan::TYPE_INHERITED , INVERT => $invert , ORDERBY => $sort_type } );
			}
		}
		if( defined( $uid ) )
		{
			$user = new Nmap::User( $nmap , $uid , { METHOD => 'id' } );
			$user->getdb;
			$tmpl = open_template( 'list-user' , 'scan' );
			$tmpl->param( 'username' => $user->get( 'username' ) );
		}
		else
		{
			$tmpl = open_template( 'list-admin' , 'scan' );

			#For each scan, we retrieve the owner username and add it to the object
			#It will be displayed
			foreach( @$scans )
			{
				$tmp = new Nmap::User( $nmap , $_->get( 'uidowner' ) , { METHOD => 'id' } );
				$tmp->getdb;
				$_->{'username'} = $tmp->get('username');
			}
		}
		if( ( defined( $scans ) ) && ( $#{@$scans} >= 0 ) )
		{
			$tmpl->param( SCANS => $scans );
			$tmpl->param( HAVE_SCANS => 1 );
			$tmpl->param( ADMIN => 1 );
		}
		output( $tmpl->output );

	}
}
elsif( $action eq 'edit' )
{
	if( $type eq 'group' )
	{
		my $group;
		my $array;
		my $optname;
		my $optval;

		$subaction = $cgi->param( 'subaction' );
		$name = $cgi->param( 'name' );

		output( undef , { WARN => 'Invalid group (name error)' } ) if( ! ( defined $name ) );


		#If the name is defined but blank, so, we have to edit the default group
		#So, we retrieve the default group through the getall method and take
		#the first value stored in the returned array
		if( $name eq "" )
		{
			$group = Nmap::Group::getall( $nmap , { DEFAULT => 1 } );
			$group = $group->[0];
		}
		else
		{
			$group = new Nmap::Group( $nmap , $name , { METHOD => 'id'} );
		}

		output( undef , { WARN => 'Invalid group (group error)' } ) if( ! ( defined $group ) );
		output( undef , { WARN => 'Invalid group (nonexistent)' } ) if( $group->getdb != 1 );


		#Here, the action is to add a right to the group
		if( ( defined $subaction ) && ( $subaction eq 'addright' ) )
		{
			$optname = $cgi->param( 'optname' );
			$optval = $cgi->param( 'optval' );

			output( undef , { WARN => 'Invalid right name' } ) if( ! ( defined $optname ) );
			output( undef , { WARN => 'Invalid right value' } ) if( ! ( defined $optval ) );

			$group->setopt( $optname , $optval );

			nmaplogevent( "add right" , $optname . " " . $optval );

			if( $group->save == 1 )
			{
				$opts = { WARN => 'Successfully add right' };
			}
			else
			{
				$opts = { WARN => 'Error while save right' };
			}
		}

		if( ( defined $subaction ) && ( $subaction eq 'delright' ) )
		{
			$optname = $cgi->param( 'optname' );
			output( undef , { WARN => 'Invalid right name' } ) if( ! ( defined $optname ) );
			$group->opt_remove( $optname );
			if( $group->save == 1 )
			{
				$opts = { WARN => 'Successfully delete right' };
			}
			else
			{
				$opts = { WARN => 'Error while delete right' };
			}
		}

		nmaplogevent( "admin" , "edit group " . $name );
		$tmpl = open_template( 'edit' , 'group' );

		$tmp = Nmap::Group::RIGHTS;
		foreach ( @{$tmp} )
		{
			push( @$array , { opt => $_ } );
		}
		$tmpl->param( ADDRIGHTS => $array );


		$tmp = Nmap::Group::RIGHTS;
		$array = undef;
		foreach ( @{$tmp} )
		{
			if( $group->opt( $_ ) == 1 )
			{
				push( @$array , { optname => $_  , optval => $group->opt_value( $_ )  , gid => $group->get( 'id' ) } );
			}
		}
		$tmpl->param( RIGHTS => $array ) if( $#{@$array} >= 0 );


		$tmpl->param( 'groupname' , $group->get( 'groupname' ) );
		$tmpl->param( 'gid' , $group->get( 'id' ) );
		output( $tmpl->output , $opts);
	}
	elsif( $type eq 'user' )
	{
		my $user;
		my $fname;
		my $group;
		my $param;

		$name = $cgi->param( 'name' );
		$fname = $cgi->param( 'firstname' );
		$user = new Nmap::User( $nmap , $name , { METHOD => 'id' } );
		$user->getdb;

		if( defined $fname )
		{
			foreach ( qw( firstname lastname state address gid) )
			{
				$param = $cgi->param( $_ );

				$user->set( $_ , $cgi->param( $_ ) )if( defined( $param ) );
				if( $user->save != 1 )
				{
					$opts = { WARN => "Bad parameter, error when save" };
				}
				$user->getdb;
			}
		}

		$user->fetch_ips();

		$group = $user->get_group;
		$user->{groupname} = $group->get( 'groupname' );
		$tmpl = open_template( 'edit' , 'user' );
		$tmpl->param( USERS => [ $user ] );
		output( $tmpl->output  , $opts);
	}
	else
	{
		output( undef , { WARN => "Unknow type" } );
	}
}
elsif( $action eq 'create' )
{
	if( $type eq 'group' )
	{
		my $group;
		$name = $cgi->param( 'groupname' );

		if( defined $name )
		{
			$group = new Nmap::Group( $nmap , $name );

			#If getdb returns 1, the group already exists
			output( open_template( 'create' , 'group' )->output  , { WARN => 'This group already exists' } ) if( $group->getdb == 1 );

			if( $group->create == 1 )
			{
				output( open_template( 'create' , 'group' )->output  , { WARN => 'Successfully created' } );
			}
			output( open_template( 'create' , 'group' )->output  , { WARN => 'Error when create the group' } );
		}
		else
		{
			output( open_template( 'create' , 'group' )->output );
		}
	}
	elsif( $type eq 'user' )
	{
		my( $mail , $pass );
		$name = $cgi->param( 'username' );
		$mail = $cgi->param( 'email' );
		$pass = $cgi->param( 'password' );

		if( defined $name )
		{
			$user = new Nmap::User( $nmap , $name );
			output( open_template( 'create' , 'user' )->output , { WARN => "Invalid username" } ) if( Nmap::CGI::Main::check_param( $name , { NOMAJ => 1 , 'MIN-LENGTH' => 2 , 'ALPHANUM' => 1 } ) ) ;
			output( open_template( 'create' , 'user' )->output , { WARN => "Error when creating user" } ) if( ! defined( $user ) );
			output( open_template( 'create' , 'user' )->output , { WARN => "User already registered" } ) if( $user->getdb );
			output( open_template( 'create' , 'user' )->output , { WARN => "Undefined password" } ) if( ! defined( $pass ) );
			output( open_template( 'create' , 'user' )->output , { WARN => "Undefined mail" } ) if( ! defined( $mail ) );
			output( open_template( 'create' , 'user' )->output , { WARN => "Invalid mail" } ) if( Nmap::CGI::Main::check_param( $mail , { EMAIL => 1 } ) ) ;

			$user->set( 'mail' , $mail );
			$user->set( 'state' , '1' );
			$user->set( 'password' , $pass );
			if( $user->create > 0 )
			{
				output( open_template( 'create' , 'user' )->output , { WARN => "Successfully created" } );
			}
		}
		else
		{
			output( open_template( 'create' , 'user' )->output );
		}
	}
}
elsif( $action eq 'setdefault' )
{
	if( $type eq 'group' )
	{
		my $groups;
		my $group;
		$name = $cgi->param( 'name' );

		output( open_template( 'list' , 'group' )->output , { WARN => 'Invalid group'}) if( ! defined( $name ) );
		
		$group = new Nmap::Group( $nmap , $name , { METHOD => 'id' } );
		$groups = Nmap::Group::getall( $nmap , { DEFAULT => 1 } );

		output( open_template( 'list' , 'group' )->output , { WARN => 'Invalid group'}) if( ! defined( $group ) );

		$group->getdb;

		foreach( @{$groups} )
		{
			$_->set( 'def' , 'f' );
			$_->save;
		}

		$group->set( 'def' , 't' );
		$group->save;

		#Finally, print the list of the groups
		$groups = Nmap::Group::getall( $nmap , { ORDERBY => 'groupname' }  );
		$tmpl = open_template( 'list' , 'group' );
		$tmpl->param( GROUPS => $groups ) if( ( defined $groups ) && ( $#{@$groups} > 0 ) );
		output( $tmpl->output , { WARN => 'Successfully change default group'});

	}
}
elsif( $action eq 'search' )
{
	if( $type eq 'user' )
	{
		$tmpl = open_template( 'search' , 'user' );
		output( $tmpl->output );
	}
	elsif( $type eq 'group' )
	{
		$tmpl = open_template( 'search' , 'group' );
		output( $tmpl->output );
	}
	elsif( $type eq 'scan' )
	{
		$tmpl = open_template( 'search' , 'scan' );
		output( $tmpl->output );
	}
}
elsif( $action eq 'delete' )
{
	if( $type eq 'user' )
	{
		my $user;
		my $scans;

		$name = $cgi->param( 'name' );
		$user = new Nmap::User( $nmap , $name , { METHOD => 'id' } );
		output( undef , { WARN => "Cannot fetch this user" } ) if( ! defined( $user ) );
		output( undef , { WARN => "Cannot fetch this user" } ) if( $user->getdb != 1 );

		$scans = Nmap::Scan::getall( $nmap , { OWNER => $user->get( 'id' ) } );
		#Delete each scan from this user
		foreach ( @$scans )
		{
			return -3 if( $_->delete != 1 );
		}

		if( $user->delete == 1 )
		{
			output( undef , { WARN => "Successfully deleted" } );
		}
		else
		{
			output( undef , { WARN => "Error when delete the user" } );
		}
	}
	elsif( $type eq 'group' )
	{
		my ( $group , $group2 );
		$name = $cgi->param( 'name' );
		$group = new Nmap::Group( $nmap , $name , { METHOD => 'id' } );
		$group2 = Nmap::Group::getall( $nmap , { DEFAULT => 'T' } );

		output( undef , { WARN => "Cannot fetch this group" } ) if( ! defined( $group ) );
		output( undef , { WARN => "Cannot fetch this group" } ) if( $group->getdb != 1 );

		if( $#{@$group2} >= 0 )
		{
			$group2 = shift( @{$group2} );
			if( $group->get( 'groupname') eq $group2->get( 'groupname' ) )
			{
				output( undef , { WARN => "You cannot delete the default group" . $group->get( 'groupname') . ":" . $group2->get( 'groupname' ) } );
			}
		}
		
		if( $group->delete == 1 )
		{
			output( undef , { WARN => "Group successfully deleted" } );
		}
		else
		{
			output( undef , { WARN => "Cannot delete this group" } );
		}
	}
	elsif( $type eq 'scan' )
	{
		my $scan;
		$name = $cgi->param( 'name' );
		$scan = new Nmap::Scan( $nmap , $name , undef , { METHOD => 'byid' } );
		output( undef , { WARN => "Cannot fetch this scan" } ) if( ! defined( $scan ) );
		output( undef , { WARN => "Cannot fetch this scan with getdb" } ) if( $scan->getdb != 1 );

		if( $scan->delete == 1 )
		{
			output( undef , { WARN => "Successfully delete scan" } );
		}
		else
		{
			output( undef , { WARN => "Cannot delete this scan" } );
		}
	}
}
nmaplogevent( "admin" , "unknown action $action" );	
output( undef , { WARN => "Unknow action" } );
