#!/usr/bin/perl
#Copyright (c) Nmap project and its contributor
#All rights reserved.

#Redistribution and use in source and binary forms, with or without 
#modification, are permitted provided that the following conditions 
#are met:

# 1. Redistributions of source code must retain the above copyright 
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in 
#    the documentation and/or other materials provided with the 
#    distribution.
# 3. Neither the name of Nmap nor the names of its contributors 
#    may be used to endorse or promote products derived from this 
#    software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.


use XML::XQL::DOM;
use XML::XQL;
use strict;

my( $odoc , $ndoc , $output );
my ( $ofile , $nfile , $output , $ofile , $nfile );

sub treat_nmaprun
{
	my( @tab1 , @tab2 );
	@tab1 = $odoc->xql( "//nmaprun" );
	@tab2 = $ndoc->xql( "//nmaprun" );
	$output .= "\t<nmaprun used=\"old\" version=\"".@tab1[0]->getAttribute("version")."\" start=\"".@tab1[0]->getAttribute("start")."\" startstr=\"".@tab1[0]->getAttribute("startstr")."\"/>\n";
	$output .= "\t<nmaprun used=\"new\" version=\"".@tab2[0]->getAttribute("version")."\"  start=\"".@tab2[0]->getAttribute("start")."\" startstr=\"".@tab2[0]->getAttribute("startstr")."\"/>\n";
}

sub treat_normal_scaninfo
{
	my $str;
	my $node;

	$node = shift;
	$str = shift;

	$output .= "<scaninfo used=\"".$str."\" type=\"".$node->getAttribute( "type" )."\" protocol=\"".$node->getAttribute( "protocol" )."\">\n";
	$output .= "\t<numservices used=\"$str\" numservices=\"".$node->getAttribute( "numservices" )."\"/>\n";
	$output .= "\t<services used=\"$str\" services=\"".$node->getAttribute( "services" )."\"/>\n";
	$output .= "</scaninfo>\n";
}



sub treat_common_scaninfo
{
	my ( $onode , $nnode );
	my ( @oservices , @nservices , @cservices , @tmp , $n , $p , $g );

	$onode = shift;
	$nnode = shift;


	$output .= "<scaninfo used=\"common\" type=\"".$nnode->getAttribute( "type" )."\" protocol=\"".$nnode->getAttribute( "protocol" )."\">\n";

	if( $onode->getAttribute( "numservices" ) == $nnode->getAttribute( "numservices" ) )
	{
		$output .= "\t<numservices used=\"common\" numservices=\"".$onode->getAttribute( "numservices" )."\"/>\n";
	}
	else
	{
		$output .= "\t<numservices used=\"new\" numservices=\"".$nnode->getAttribute( "numservices" )."\"/>\n";
		$output .= "\t<numservices used=\"old\" numservices=\"".$onode->getAttribute( "numservices" )."\"/>\n";
	}

	@oservices = split( /\s*,\s*/ , $onode->getAttribute( "services" ) );
	@nservices = split( /\s*,\s*/ , $nnode->getAttribute( "services" ) );

	foreach $n ( @oservices )
	{
		foreach $p ( @nservices )
		{
			if( $n == $p )
			{
				push( @cservices , $n );
			}
		}
	}

	foreach $n ( @oservices )
	{
		$g = 0;
		foreach $p ( @cservices )
		{
			$g = 1 if( $n == $p );

		}
		if( $g == 0 )
		{
			push( @tmp , $n );
		}
	}
	@oservices = @tmp;


	#Flush @tmp
	foreach ( @tmp )
	{
		pop( @tmp );
	}

	foreach $n ( @nservices )
	{
		$g = 0;
		foreach $p ( @cservices )
		{
			$g = 1 if( $n == $p );

		}
		if( $g == 0 )
		{
			push( @tmp , $n );
		}
	}
	@nservices = @tmp;

	if( $#cservices != -1 )
	{
		$output .= "\t<services used=\"common\" services=\"";
		while( $p = pop( @cservices ) )
		{
			$output .= "$p";
			$output .= "," if( $#cservices != -1 );
		}
		$output .= "\"/>\n";
	}

	if( $#oservices != -1 )
	{
		$output .= "\t<services used=\"old\" services=\"";
		while( $p = pop( @oservices ) )
		{
			$output .= "$p";
			$output .= "," if( $#oservices != -1 );
		}
		$output .= "\"/>\n";
	}

	if( $#nservices != -1 )
	{
		$output .= "\t<services used=\"new\" services=\"";
		while( $p = pop( @nservices ) )
		{
			$output .= "$p";
			$output .= "," if( $#nservices != -1 );
		}
		$output .= "\"/>\n";
	}
	$output .= "</scaninfo>\n";
}

#Will compare scaninfos for two XML files
#Browse all scaninfo, see that is different, what is in common
sub compare_scaninfo
{
	my ( @tab1 , @tab2 );
	my ( $node1 , $node2 );
	my $query;

	@tab1 = $odoc->xql( "//scaninfo" );

	foreach $node1 ( @tab1 )
	{
		$query= "//scaninfo[\@type='".$node1->getAttribute( "type" )."']";
		@tab2 = $ndoc->xql( $query );

		if( $#tab2 >= 0 )
		{
			treat_common_scaninfo( $node1 , $tab2[0] );
		}
		else
		{
			treat_normal_scaninfo( $node1 , "old" );
		}
	}

	@tab1 = $ndoc->xql( "//scaninfo" );
	foreach $node1 ( @tab1 )
	{
		$query = "//scaninfo[\@type='".$node1->getAttribute( "type" )."']";
		@tab2 = $odoc->xql( $query );

		if( $#tab2 == -1 )
		{
			treat_normal_scaninfo( $node1 , "new" );
		}
	}

}

sub treat_common_serviceport
{
	my $onode = shift;
	my $nnode = shift;

	$output .= "\t\t<service used=\"common\" name=\"" .$onode->getAttribute( "name" ). "\">\n";

	if( $onode->getAttribute( "product" ) eq $nnode->getAttribute( "product" ) )
	{
		$output .= "\t\t\t<product used=\"common\" product=\"" .$onode->getAttribute( "product" ). "\"/>\n" 			if( ( defined( $onode->getAttribute( "product" ) ) ) && ( $onode->getAttribute( "product" ) ne '' ) );
	}
	else
	{
		$output .= "\t\t\t<product used=\"old\" product=\"" .$onode->getAttribute( "product" ). "\"/>\n" 			if( ( defined( $onode->getAttribute( "product" ) ) ) && ( $onode->getAttribute( "product" ) ne '' ) );
		$output .= "\t\t\t<product used=\"new\" product=\"" .$nnode->getAttribute( "product" ). "\"/>\n" 			if( ( defined( $nnode->getAttribute( "product" ) ) ) && ( $nnode->getAttribute( "product" ) ne '' ) );
	}



	if( $onode->getAttribute( "extrainfo" ) eq $nnode->getAttribute( "extrainfo" ) )
	{
		$output .= "\t\t\t<extrainfo used=\"common\" product=\"" .$onode->getAttribute( "extrainfo" ). "\"/>\n" 			if( ( defined( $onode->getAttribute( "extrainfo" ) ) ) && ( $onode->getAttribute( "extrainfo" ) ne '' ) );
	}
	else
	{
		$output .= "\t\t\t<extrainfo used=\"old\" extrainfo=\"" .$onode->getAttribute( "extrainfo" ). "\"/>\n" 			if( ( defined( $onode->getAttribute( "extrainfo" ) ) ) && ( $onode->getAttribute( "extrainfo" ) ne '' ) );
		$output .= "\t\t\t<extrainfo used=\"new\" extrainfo=\"" .$nnode->getAttribute( "extrainfo" ). "\"/>\n" 			if( ( defined( $nnode->getAttribute( "extrainfo" ) ) ) && ( $nnode->getAttribute( "extrainfo" ) ne '' ) );
	}

	if( $onode->getAttribute( "method" ) eq $nnode->getAttribute( "method" ) )
	{
		$output .= "\t\t\t<method used=\"common\" method=\"" .$onode->getAttribute( "method" ). "\"/>\n" 			if( ( defined( $onode->getAttribute( "method" ) ) ) && ( $onode->getAttribute( "method" ) ne '' ) );
	}
	else
	{
		$output .= "\t\t\t<method used=\"old\" method=\"" .$onode->getAttribute( "method" ). "\"/>\n" 			if( ( defined( $onode->getAttribute( "method" ) ) ) && ( $onode->getAttribute( "method" ) ne '' ) );
		$output .= "\t\t\t<method used=\"new\" method=\"" .$nnode->getAttribute( "method" ). "\"/>\n" 			if( ( defined( $nnode->getAttribute( "method" ) ) ) && ( $nnode->getAttribute( "method" ) ne '' ) );
	}



	if( $onode->getAttribute( "conf" ) eq $nnode->getAttribute( "conf" ) )
	{
		$output .= "\t\t\t<conf used=\"common\" conf=\"" .$onode->getAttribute( "conf" ). "\"/>\n" 			if( ( defined( $onode->getAttribute( "conf" ) ) ) && ( $onode->getAttribute( "conf" ) ne '' ) );
	}
	else
	{
		$output .= "\t\t\t<conf used=\"old\" conf=\"" .$onode->getAttribute( "conf" ). "\"/>\n" 			if( ( defined( $onode->getAttribute( "conf" ) ) ) && ( $onode->getAttribute( "conf" ) ne '' ) );
		$output .= "\t\t\t<conf used=\"new\" conf=\"" .$nnode->getAttribute( "conf" ). "\"/>\n" 			if( ( defined( $nnode->getAttribute( "conf" ) ) ) && ( $nnode->getAttribute( "conf" ) ne '' ) );
	}


	$output .= "\t\t</service>\n";
}

sub treat_common_port
{
	my ( $oport , $nport );
	my ( @tab1 , @tab2 );
	$oport = shift;
	$nport = shift;

	$output .= "\t<port protocol=\"".$oport->getAttribute( "protocol" )."\" portid=\"".$oport->getAttribute( "portid" )."\" used=\"common\">\n";
	@tab1 = $oport->getElementsByTagName( "state" );
	@tab2 = $nport->getElementsByTagName( "state" );

	#Twice <port/> have state element
	if( ( $#tab1 != -1 ) &&
		( $#tab2 != -1 ) )
		{
			if( @tab1[0]->getAttribute( "state" ) eq @tab2[0]->getAttribute( "state" ) )
			{
				$output .= "\t\t<state used=\"common\" state=\"".@tab1[0]->getAttribute( "state" )."\"/>\n";
			}
			else
			{
				$output .= "\t\t<state used=\"old\" state=\"".@tab1[0]->getAttribute( "state" )."\"/>\n";
				$output .= "\t\t<state used=\"new\" state=\"".@tab2[0]->getAttribute( "state" )."\"/>\n";
			}
		}
	@tab1 = $oport->getElementsByTagName( "service" );
	@tab2 = $nport->getElementsByTagName( "service" );

	if( ( $#tab1 == -1 ) && ( $#tab2 > -1 ) )
	{
		treat_normal_serviceport( @tab2[0] , "new" );
	}
	elsif( ( $#tab1 > -1 ) && ( $#tab2 == -1 ) )
	{
		treat_normal_serviceport( @tab1[0] , "old" );
	}
	elsif(  ( $#tab1 > -1 ) && ( $#tab2 > -1 ) )
	{
		if( @tab1[0]->getAttribute( "name" ) eq @tab2[0]->getAttribute( "name" ) )
		{
			treat_common_serviceport( @tab1[0] , @tab2[0] );
		}
		else
		{
			treat_normal_serviceport( @tab1[0] , "old" );
			treat_normal_serviceport( @tab2[0] , "new" );
		}
	}

	$output .="\t</port>\n"
}

sub treat_normal_port
{
	my $port;
	my $state;
	my (@tab1 , @tab2 );

	$port = shift;
	$state = shift;

	$output .= "\t<port protocol=\"".$port->getAttribute( "protocol" )."\" portid=\"".$port->getAttribute( "portid" )."\" used=\"".$state."\">\n";
	@tab1 = $port->getElementsByTagName( "state" );
	$output .= "\t\t<state used=\"".$state."\" state=\"".@tab1[0]->getAttribute( "state" )."\"/>\n";
	@tab1 = $port->getElementsByTagName( "service" );
	if( $#tab1 != -1 )
	{
		treat_normal_serviceport( shift( @tab1 ) , $state );
	}
	$output .="\t</port>\n"
}

sub treat_normal_serviceport
{
	my $node = shift;
	my $state = shift;
	$output .= "\t\t<service used=\"".$state."\" name=\"" .$node->getAttribute( "name" ). "\">\n";
	$output .= "\t\t\t<product used=\"".$state."\" product=\"" .$node->getAttribute( "product" ). "\"/>\n" 			if( ( defined( $node->getAttribute( "product" ) ) ) && ( $node->getAttribute( "product" ) ne '' ) );
	$output .= "\t\t\t<extrainfo used=\"".$state."\" extrainfo=\"" .$node->getAttribute( "extrainfo" ). "\"/>\n"		if( ( defined( $node->getAttribute( "extrainfo" ) ) ) && ( $node->getAttribute( "extrainfo" ) ne '' ) );
	$output .= "\t\t\t<method used=\"".$state."\" method=\"" .$node->getAttribute( "method" ). "\"/>\n"				if( ( defined( $node->getAttribute( "method" ) ) ) && (   $node->getAttribute( "method" ) ne '' ) );
	$output .= "\t\t\t<conf used=\"".$state."\" conf=\"" .$node->getAttribute( "conf" ). "\"/>\n"						if( ( defined( $node->getAttribute( "conf" ) ) ) && (  $node->getAttribute( "conf" ) ne '' ) );
	$output .= "\t\t</service>\n";
}


sub treat_common_host
{
	my	$onode;
	my	$nnode;
	my	( @tab1 , @tab2 , @tab );
	my  ( $tmp , $tmp2 , $found , $query , $port );

	$onode = shift;
	$nnode = shift;

	$output .= "<host used=\"common\">\n";



	@tab1 = $onode->xql( "address" );
	@tab2 = $nnode->xql( "address" );

	foreach $tmp ( @tab1 )
	{
		$found = 0;
		foreach $tmp2 ( @tab2 )
		{
			if( $tmp->getAttribute( "addr" ) eq $tmp2->getAttribute( "addr" ) )
			{
				$found = 1;
			}
		}

		if( $found == 0 )
		{
			$output .= "\t<address used=\"old\" addr=\"".$tmp->getAttribute("addr")."\" addrtype=\"".$tmp->getAttribute("addrtype")."\"/>\n";
		}
		else
		{
			$output .= "\t<address used=\"common\" addr=\"".$tmp->getAttribute("addr")."\" addrtype=\"".$tmp->getAttribute("addrtype")."\"/>\n";
		}
	}
	foreach $tmp ( @tab1 )
	{
		$found = 0;
		foreach $tmp2 ( @tab2 )
		{
			if( $tmp->getAttribute( "addr" ) eq $tmp2->getAttribute( "addr" ) )
			{
				$found = 1;
			}
		}
		if( $found == 0 )
		{
			$output .= "\t<address used=\"new\" addr=\"".$tmp->getAttribute("addr")."\" addrtype=\"".$tmp->getAttribute("addrtype")."\"/>\n";
		}
	}

	undef( @tab1 );
	undef( @tab2 );

	@tab1 = $onode->xql( "status" );
	@tab2 = $nnode->xql( "status" );

	if( @tab1[0]->getAttribute( "state" ) eq @tab2[0]->getAttribute( "state" ) )
	{
		$output .= "\t<status used=\"common\" state=\"".@tab1[0]->getAttribute( "state" )."\"/>\n";
	}
	else
	{
		$output .= "\t<status used=\"old\" state=\"".@tab1[0]->getAttribute( "state" )."\"/>\n";
		$output .= "\t<status used=\"new\" state=\"".@tab2[0]->getAttribute( "state" )."\"/>\n";
	}


	$output .= "\t<ports>\n";
	$query = "ports/port";
	@tab = $onode->xql( $query );
	foreach $port ( @tab )
	{
		$query = "ports/port[\@portid = '".$port->getAttribute('portid')."' and \@protocol='".$port->getAttribute('protocol')."']";
		@tab2 = $nnode->xql( $query );
		if( $#tab2 != -1  )
		{
			treat_common_port( $port , @tab2[0] );
		}
		else
		{
			treat_normal_port( $port , "old" );
		}
	}

	$query = "ports/port";
	@tab = $nnode->xql( $query );

	foreach $port ( @tab )
	{
		$query = "ports/port[\@portid = '".$port->getAttribute('portid')."' and \@protocol='".$port->getAttribute('protocol')."']";
		@tab2 = $onode->xql( $query );
		if( $#tab2 == -1  )
		{
			treat_normal_port( $port , "new" );
		}
	}
	$output .= "\t</ports>\n";
	$output .= "</host>\n";
}


sub treat_normal_host
{
	my 	$node;
	my	$state; 
	my 	( $tmp , @tab , $query , $port );
	$node	= shift;
	$state	= shift;

	$output .= "<host used=\"".$state."\">\n";


	@tab = $node->xql( "address" );

	foreach $tmp ( @tab )
	{
		$output .= "\t<address used=\"".$state."\" addr=\"".$tmp->getAttribute("addr")."\" addrtype=\"".$tmp->getAttribute("addrtype")."\"/>\n";
	}

	@tab = $node->xql( "status" );


	$output .= "\t<status used=\"".$state."\" state=\"".@tab[0]->getAttribute( "state" )."\"/>\n";


	$query = "ports/port";
	@tab = $node->xql( $query );

	$output .= "\t<ports>\n";
	foreach $port ( @tab )
	{
		treat_normal_port( $port , $state );
	}

	$output .= "\t</ports>\n";
	$output .= "</host>\n";
}


sub compare_hosts
{
	my ( @tab1 , @tab2 );
	my ( $node1 , $node2 , $query );


	#First, fetch common hosts
	$query = "//host[address/\@addr=document('".$nfile."')//host/address/\@addr]";
	@tab1 = $odoc->xql( $query );

	foreach $node1 ( @tab1 )
	{
		treat_common_host( $node1 , ($ndoc->xql( $query ))[0] );
	}


	#Retrieve hosts only in the old document
	$query = "//host[address/\@addr!=document('".$nfile."')//host/address/\@addr]";
	@tab1 = $odoc->xql( $query );

	foreach $node1 ( @tab1 )
	{
		treat_normal_host( $node1 , "old" );
	}

	#Retrieve hosts only in new document
	$query = "//host[address/\@addr!=document('".$ofile."')//host/address/\@addr]";
	@tab1 = $ndoc->xql( $query );

	foreach $node1 ( @tab1 )
	{
		treat_normal_host( $node1 , "new" );
	}

}


sub display_intro
{
	$output .= "<?xml version=\"1.0\"?>\n";
	$output .= "<?xml-stylesheet href=\"/usr/share/nmap-cgi/nmap-cgi.xsl\" type=\"text/xsl\"?>\n";
	$output .= "<nmapcompare>\n";
}

sub display_outro
{
	$output .= "</nmapcompare>\n";
}





sub compare_files
{
	my	( $output , $parser );
	$ofile = shift;
	$nfile = shift;

	exit( 1 ) if( ! -f $ofile );
	exit( 1 ) if( ! -f $nfile );

	$parser = new XML::DOM::Parser;
	$odoc = $parser->parsefile( $ofile );
	$ndoc = $parser->parsefile( $nfile );
	$output = "";
	display_intro( );
	treat_nmaprun( );
	compare_scaninfo( );
	compare_hosts();
	display_outro( );
}

if( $#ARGV != 1 )
{
	print "Use this tool as it: nmap-xml-diff file1.xml file2.xml\n";
	exit( 1 );
}

compare_files( $ARGV[0] , $ARGV[1] );
print $output;
exit( 0 );
